library ac_dart.host_service.bitbucket.impl;

import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:atlassian_connect_host/src/host_service/bitbucket/bitbucket_service.dart';
import 'package:atlassian_connect_host/src/host_service/host_http_client.dart';
import 'package:option/option.dart';
import 'package:shelf_oauth/shelf_oauth.dart';

class BitbucketServiceImpl implements BitbucketService {
  final AcHostHttpClient _client;

  BitbucketServiceImpl(this._client);

  Future<OAuth2TokenPair> obtainOAuthTokenFromJwt() async {
    final request =
        await _client.post('site/oauth2/access_token', signRequest: true);

    request.headers
        .add(HttpHeaders.CONTENT_TYPE, 'application/x-www-form-urlencoded');
    request.headers.add(HttpHeaders.ACCEPT, 'application/json');
    print(request.headers);
    request.write("grant_type=urn:bitbucket:oauth2:jwt");
    final response = await request.close();
//        {'grant_type': 'urn:bitbucket:oauth2:jwt'},
//        headers: {"Authorization": "JWT $jwtToken"});

    print(response.statusCode);
    print(response.headers);
    final body = await response.transform(UTF8.decoder).take(1).first;
    print(body);

    /**
     * TODO: fix OAuth2TokenPair to/from Json as it is using camel case whereas
     * the spec uses snake case
     */
    final m = JSON.decode(body);
    final accessToken = m['access_token'];
    final refreshToken = new Option(m['refresh_token']);
    final expiresIn = new Option(m['expires_in']);
    final expires = expiresIn
        .map((expIn) => new DateTime.now().add(new Duration(seconds: expIn)));
    final grantedScopes = new Option(_toList(m['scopes']));

    return new OAuth2TokenPair(
        accessToken, refreshToken, expires, grantedScopes);
  }
}

// sadly bitbucket returns a string if it is a single value
List<String> _toList(value) {
  if (value == null) {
    return const [];
  } else if (value is Iterable) {
    return value.toList();
  } else {
    return [value.toString()];
  }
}
